unit uMHA_AUX_ISMI;

interface

uses
  SysUtils;

function ScoreAnswers_ISMI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_ISMI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The ISMI contains 29 items which produce 5 subscale scores and a total score.
  Each score is calculated by adding the item scores together and then dividing
  by the total number of answered items. If any items are not answered, the
  total number to be divided by is reduced. The resulting score should range
  from 1-4. For example, if someone answers 5 of the 6 Alienation items, the
  Alienation score is produced by adding together the 5 answered items and
  dividing by 5.

*}



function ScoreAnswers_ISMI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  Count, i, Answer,
    Alien, Stereo, Discrim, Withdraw, Resist: Integer;

  AlienSum, StereoSum, DiscrimSum, WithdrawSum, ResistSum,
    TotalSum
                              : Extended;
  TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // Output:  ~ScaleName^ScaleIEN^ScaleScore~

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'ISMI' then
  begin
     Result := 'YES';
     exit;
  end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  Alien := 0;
  Stereo := 0;
  Discrim := 0;
  Withdraw := 0;
  Resist := 0;

  AlienSum := 0;
  StereoSum := 0;
  DiscrimSum := 0;
  WithdrawSum := 0;
  ResistSum := 0;
  TotalSum := 0;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = NOT_ASKED) or
        (TempAnswer = SKIPPED) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          TempAnswer := '0';
        end
      else
        // Tally the scale's sum and total sum
        begin
          Answer := StrToIntDef(TempAnswer, 0) + 1;
          // Alien
          if (i = 1) then
            begin
              Inc(Alien);
              AlienSum := AlienSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 5) then
            begin
              Inc(Alien);
              AlienSum := AlienSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 8) then
            begin
              Inc(Alien);
              AlienSum := AlienSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 16) then
            begin
              Inc(Alien);
              AlienSum := AlienSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 17) then
            begin
              Inc(Alien);
              AlienSum := AlienSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 21) then
            begin
              Inc(Alien);
              AlienSum := AlienSum + Answer;
              TotalSum := TotalSum + Answer;
            end;


          // Stereotypes
          if (i = 2) then
            begin
              Inc(Stereo);
              StereoSum := StereoSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 6) then
            begin
              Inc(Stereo);
              StereoSum := StereoSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 10) then
            begin
              Inc(Stereo);
              StereoSum := StereoSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 18) then
            begin
              Inc(Stereo);
              StereoSum := StereoSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 19) then
            begin
              Inc(Stereo);
              StereoSum := StereoSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 23) then
            begin
              Inc(Stereo);
              StereoSum := StereoSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 29) then
            begin
              Inc(Stereo);
              StereoSum := StereoSum + Answer;
              TotalSum := TotalSum + Answer;
            end;

          // Discrimination
          if (i = 3) then
            begin
              Inc(Discrim);
              DiscrimSum := DiscrimSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 15) then
            begin
              Inc(Discrim);
              DiscrimSum := DiscrimSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 22) then
            begin
              Inc(Discrim);
              DiscrimSum := DiscrimSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 25) then
            begin
              Inc(Discrim);
              DiscrimSum := DiscrimSum + Answer;
              TotalSum := TotalSum + Answer;
            end;

          if (i = 28) then
            begin
              Inc(Discrim);
              DiscrimSum := DiscrimSum + Answer;
              TotalSum := TotalSum + Answer;
            end;

          // Social Withdrawal
          if (i = 4) then
            begin
              Inc(Withdraw);
              WithdrawSum := WithdrawSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 9) then
            begin
              Inc(Withdraw);
              WithdrawSum := WithdrawSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 11) then
            begin
              Inc(Withdraw);
              WithdrawSum := WithdrawSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 12) then
            begin
              Inc(Withdraw);
              WithdrawSum := WithdrawSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 13) then
            begin
              Inc(Withdraw);
              WithdrawSum := WithdrawSum + Answer;
              TotalSum := TotalSum + Answer;
            end;
          if (i = 20) then
            begin
              Inc(Withdraw);
              WithdrawSum := WithdrawSum + Answer;
              TotalSum := TotalSum + Answer;
            end;

          // Stigma Resistance: reverse score before adding to total
          if (i = 7) then
            begin
              Inc(Resist);
              ResistSum := ResistSum + (5 - Answer);
              TotalSum := TotalSum + (5 - Answer);
            end;
          if (i = 14) then
            begin
              Inc(Resist);
              ResistSum := ResistSum + (5 - Answer);
              TotalSum := TotalSum + (5 - Answer);
            end;
          if (i = 24) then
            begin
              Inc(Resist);
              ResistSum := ResistSum + (5 - Answer);
              TotalSum := TotalSum + (5 - Answer);
            end;
          if (i = 26) then
            begin
              Inc(Resist);
              ResistSum := ResistSum + (5 - Answer);
              TotalSum := TotalSum + (5 - Answer);
            end;
          if (i = 27) then
            begin
              Inc(Resist);
              ResistSum := ResistSum + (5 - Answer);
              TotalSum := TotalSum + (5 - Answer);
            end;
        end; // Tally scale's sum
    end; // for i

  // ~ScaleName^ScaleIEN^ScaleScore~
  if Alien <> 0 then
    begin
      AlienSum := AlienSum / Alien;
      Result := Spacer + 'Alienation^^' + FloatToStrF(AlienSum, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Spacer + 'Alienation^^All questions skipped' + RECORD_END;

  if Stereo <> 0 then
    begin
      StereoSum := StereoSum / Stereo;
      Result := Result + Spacer + 'Stereotype Endorsement^^' +
        FloatToStrF(StereoSum, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Spacer + 'Stereotype Endorsement^^All questions skipped' + RECORD_END;

  if Discrim <> 0 then
    begin
      DiscrimSum := DiscrimSum / Discrim;
      Result := Result + Spacer + 'Discrimination Experience^^' +
        FloatToStrF(DiscrimSum, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Spacer + 'Discrimination Experience^^All questions skipped' + RECORD_END;

  if WithDraw <> 0 then
    begin
      WithDrawSum := WithDrawSum / WithDraw;
      Result := Result + Spacer + 'Social Withdrawal^^' +
        FloatToStrF(WithdrawSum, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Spacer + 'Social Withdrawal^^All questions skipped' + RECORD_END;

  if Resist <> 0 then
    begin
      ResistSum := ResistSum / Resist;
      Result := Result + Spacer + 'Stigma Resistance^^' +
        FloatToStrF(ResistSum, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Spacer + 'Stigma Resistance^^All questions skipped' + RECORD_END;

  TotalSum := TotalSum / (Alien + Stereo + Discrim + WithDraw + Resist);
  Result := Result + Spacer + 'Total Score^^' + FloatToStrF(TotalSum, ffFixed, 8, 2) + RECORD_END;
end;


function ProgressNote_ISMI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer, questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := Indent + 'ISMI Scales' + #13;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) do
    begin
      TempStr := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      while Length(TempStr) < 34 do // Acts as a tab for spacing
        TempStr := TempStr + ' ';
      Result := Result + Indent + TempStr +
        Piece(Piece(ScoreString, RECORD_END, i), U, 3) + #13;
    end;

  Result := Result + #13 + Indent +
    '4-category method (following the method used by Lysaker et al., 2007)' + #13 + Indent +
    '1.00-2.00: 	minimal to no internalized stigma' + #13 + Indent +
    '2.01-2.50: 	mild internalized stigma' + #13 + Indent +
    '2.51-3.00: 	moderate internalized stigma' + #13 + Indent +
    '3.01-4.00:	severe internalized stigma' + #13 + #13 + Indent +

  '2-category method (following the method used by Ritsher & Phelan, 2004)' + #13 + Indent +
    '1.00-2.50: 	does not report high internalized stigma' + #13 + Indent +
    '2.51-4.00:	reports high internalized stigma';

  Result := Result + #13; //+ #13 +
  //  Indent + '------------ ISMI Responses ----------' + #13 + #13;

end;

end.

