unit uMHA_AUX_NEOPI3;

interface

uses
Classes,
  SysUtils;

function ScoreAnswers_NEOPI3(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_NEOPI3(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;


implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The NEO-PI-3 has five scale scores, each having six sub scales.


*}

function GetTScore(Scale: string; raw: integer): integer;
// T-score range 20-80.
// Formula:  t = New Mean (50) + New SD (10) * (Scale Mean - Raw Score) / Scale SD
// Scale Mean and Scale SD are for combined male & female norms.
// See page 114 in manual for values.

begin

  result := -1;

  if raw = -1 then
    exit;

  if Scale = 'N' then
    result := Round(50 + 10 * ((82.7 - raw) / 22.3));
  if Scale = 'E' then
    result := Round(50 + 10 * ((110.4 - raw) / 19.3));
  if Scale = 'O' then
    result := Round(50 + 10 * ((107.7 - raw) / 18.6));
  if Scale = 'A' then
    result := Round(50 + 10 * ((119.1 - raw) / 18.2));
  if Scale = 'C' then
    result := Round(50 + 10 * ((121.1 - raw) / 19.9));

  if Scale = 'N1' then
    result := Round(50 + 10 * ((15.7 - raw) / 5.6));
  if Scale = 'N2' then
    result := Round(50 + 10 * ((13.6 - raw) / 4.7));
  if Scale = 'N3' then
    result := Round(50 + 10 * ((13.3 - raw) / 5.5));
  if Scale = 'N4' then
    result := Round(50 + 10 * ((13.4 - raw) / 5.0));
  if Scale = 'N5' then
    result := Round(50 + 10 * ((15.7 - raw) / 4.2));
  if Scale = 'N6' then
    result := Round(50 + 10 * ((11.1 - raw) / 4.3));

  if Scale = 'E1' then
    result := Round(50 + 10 * ((22.3 - raw) / 4.4));
  if Scale = 'E2' then
    result := Round(50 + 10 * ((17.1 - raw) / 5.1));
  if Scale = 'E3' then
    result := Round(50 + 10 * ((16.1 - raw) / 4.9));
  if Scale = 'E4' then
    result := Round(50 + 10 * ((17.7 - raw) / 4.3));
  if Scale = 'E5' then
    result := Round(50 + 10 * ((17.2 - raw) / 5.1));
  if Scale = 'E6' then
    result := Round(50 + 10 * ((20.2 - raw) / 4.9));

  if Scale = 'O1' then
    result := Round(50 + 10 * ((17.1 - raw) / 4.6));
  if Scale = 'O2' then
    result := Round(50 + 10 * ((16.4 - raw) / 5.7));
  if Scale = 'O3' then
    result := Round(50 + 10 * ((20.4 - raw) / 4.1));
  if Scale = 'O4' then
    result := Round(50 + 10 * ((15.9 - raw) / 3.6));
  if Scale = 'O5' then
    result := Round(50 + 10 * ((17.8 - raw) / 5.4));
  if Scale = 'O6' then
    result := Round(50 + 10 * ((20.1 - raw) / 4.2));

  if Scale = 'A1' then
    result := Round(50 + 10 * ((19.5 - raw) / 4.5));
  if Scale = 'A2' then
    result := Round(50 + 10 * ((20.3 - raw) / 4.7));
  if Scale = 'A3' then
    result := Round(50 + 10 * ((23.4 - raw) / 3.9));
  if Scale = 'A4' then
    result := Round(50 + 10 * ((16.4 - raw) / 4.5));
  if Scale = 'A5' then
    result := Round(50 + 10 * ((19.1 - raw) / 4.4));
  if Scale = 'A6' then
    result := Round(50 + 10 * ((20.4 - raw) / 3.9));

  if Scale = 'C1' then
    result := Round(50 + 10 * ((21.4 - raw) / 3.8));
  if Scale = 'C2' then
    result := Round(50 + 10 * ((19.3 - raw) / 5.0));
  if Scale = 'C3' then
    result := Round(50 + 10 * ((22.3 - raw) / 4.0));
  if Scale = 'C4' then
    result := Round(50 + 10 * ((19.6 - raw) / 4.5));
  if Scale = 'C5' then
    result := Round(50 + 10 * ((20.5 - raw) / 4.6));
  if Scale = 'C6' then
    result := Round(50 + 10 * ((17.9 - raw) / 4.5));

  if result = -1 then
    exit;

  if result > 80 then
    result := 80;

  if result < 20 then
    result := 20;
end;

function ScoreAnswers_NEOPI3(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';
  REVERSE                     = 'REVERSE';
  REGULAR                     = 'REGULAR';

var
  Count, i,
    Skip, V0, V1, V2, V3, V4,
    Zero, One, Two, Three, Four,
    N, N1, N2, N3, N4, N5, N6,
    E, E1, E2, E3, E4, E5, E6,
    O, O1, O2, O3, O4, O5, O6,
    A, A1, A2, A3, A4, A5, A6,
    C, C1, C2, C3, C4, C5, C6 : Integer;

  TempResult,
    VA, VB, VC, VR,
    TempAnswer                : string;

  procedure ProcessAnswer(var Scale: integer; Answer: string; RegOrRev: string);
  var
    ans                       : string;
  begin
    Ans := Answer;

    if Answer = '-1' then // score a blank as neutral
      begin
        Ans := '2';
        V0 := 0;
        V1 := 0;
        V2 := 0;
        V3 := 0;
        V4 := 0;
      end;
    if Answer = '0' then // Stronly Disagree
      begin
        Inc(Zero);
        Inc(V0);
        V1 := 0;
        V2 := 0;
        V3 := 0;
        V4 := 0;
      end;
    if Answer = '1' then // Disagree
      begin
        Inc(One);
        V0 := 0;
        Inc(V1);
        V2 := 0;
        V3 := 0;
        V4 := 0;
      end;
    if Answer = '2' then // Neutral
      begin
        Inc(Two);
        V0 := 0;
        V1 := 0;
        Inc(V2);
        V3 := 0;
        V4 := 0;
      end;
    if Answer = '3' then // Agree
      begin
        Inc(Three);
        V0 := 0;
        V1 := 0;
        V2 := 0;
        Inc(V3);
        V4 := 0;
      end;
    if Answer = '4' then // Strongly Agree
      begin
        Inc(Four);
        V0 := 0;
        V1 := 0;
        V2 := 0;
        V3 := 0;
        Inc(V4);
      end;

    if RegOrRev = REVERSE then
      begin
        Scale := Scale + 4 - StrToIntDef(Ans, 0);

      end
    else
      begin
        Scale := Scale + StrToIntDef(Ans, 0);

      end;

    if (V0 = 7) then
      VR := VR + 'Strongly disagree has been checked at least six times in a row. ';
    if (V1 = 10) then
      VR := VR + 'Disagree has been checked at least nine times in a row. ';
    if (V2 = 11) then
      VR := VR + 'Neutral has been checked at least ten times in a row. ';
    if (V3 = 15) then
      VR := VR + 'Agree has been checked at least 14 times in a row. ';
    if (V4 = 10) then
      VR := VR + 'Strongly agree has been checked at least nine times in a row. ';

  end;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll

  if AnswerString = 'NEO-PI-3' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  N1 := 0;
  N2 := 0;
  N3 := 0;
  N4 := 0;
  N5 := 0;
  N6 := 0;
  N := 0;
  E1 := 0;
  E2 := 0;
  E3 := 0;
  E4 := 0;
  E5 := 0;
  E6 := 0;
  E := 0;
  O1 := 0;
  O2 := 0;
  O3 := 0;
  O4 := 0;
  O5 := 0;
  O6 := 0;
  O := 0;
  A1 := 0;
  A2 := 0;
  A3 := 0;
  A4 := 0;
  A5 := 0;
  A6 := 0;
  A := 0;
  C1 := 0;
  C2 := 0;
  C3 := 0;
  C4 := 0;
  C5 := 0;
  C6 := 0;
  C := 0;
  //07/31/2012 changed -1 to 0 llh
  Skip := 0;
  Zero := 0;
  One := 0;
  Two := 0;
  Three := 0;
  Four := 0;

  VA := '';
  VB := '';
  VC := '';
  VR := '';
  V0 := 0;
  V1 := 0;
  V2 := 0;
  V3 := 0;
  V4 := 0;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);
     
      // Count the number of skipped (Blank) answers for each scale
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = NOT_ASKED) or
        (TempAnswer = SKIPPED) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) then
        begin
          TempAnswer := 'X';
          Inc(Skip);
        end // Skipped questions
      else
        begin
          // N1
          if (i = 31) or (i = 91) or (i = 151) or (i = 211) then
            ProcessAnswer(N1, TempAnswer, REGULAR);
          if (i = 1) or (i = 61) or (i = 121) or (i = 181) then
            ProcessAnswer(N1, TempAnswer, REVERSE);
          // E1
          if (i = 2) or (i = 62) or (i = 122) or (i = 152) or (i = 182) or (i = 212) then
            ProcessAnswer(E1, TempAnswer, REGULAR);
          if (i = 32) or (i = 92) then
            ProcessAnswer(E1, TempAnswer, REVERSE);
          // O1
          if (i = 3) or (i = 63) or (i = 123) then
            ProcessAnswer(O1, TempAnswer, REGULAR);
          if (i = 33) or (i = 93) or (i = 153) or (i = 183) or (i = 213) then
            ProcessAnswer(O1, TempAnswer, REVERSE);
          // A1
          if (i = 34) or (i = 94) or (i = 154) or (i = 184) or (i = 214) then
            ProcessAnswer(A1, TempAnswer, REGULAR);
          if (i = 4) or (i = 64) or (i = 124) then
            ProcessAnswer(A1, TempAnswer, REVERSE);
          // C1
          if (i = 5) or (i = 65) or (i = 125) or (i = 185) or (i = 215) then
            ProcessAnswer(C1, TempAnswer, REGULAR);
          if (i = 35) or (i = 95) or (i = 155) then
            ProcessAnswer(C1, TempAnswer, REVERSE);

          // N2
          if (i = 6) or (i = 66) or (i = 126) or (i = 186) or (i = 216) then
            ProcessAnswer(N2, TempAnswer, REGULAR);
          if (i = 36) or (i = 96) or (i = 156) then
            ProcessAnswer(N2, TempAnswer, REVERSE);
          // E2
          if (i = 37) or (i = 97) or (i = 157) or (i = 217) then
            ProcessAnswer(E2, TempAnswer, REGULAR);
          if (i = 7) or (i = 67) or (i = 127) or (i = 187) then
            ProcessAnswer(E2, TempAnswer, REVERSE);
          // O2
          if (i = 38) or (i = 98) or (i = 158) or (i = 188) or (i = 218) then
            ProcessAnswer(O2, TempAnswer, REGULAR);
          if (i = 8) or (i = 68) or (i = 128) then
            ProcessAnswer(O2, TempAnswer, REVERSE);
          // A2
          if (i = 9) or (i = 69) or (i = 129) then
            ProcessAnswer(A2, TempAnswer, REGULAR);
          if (i = 39) or (i = 99) or (i = 159) or (i = 189) or (i = 219) then
            ProcessAnswer(A2, TempAnswer, REVERSE);
          // C2
          if (i = 40) or (i = 100) or (i = 160) then
            ProcessAnswer(C2, TempAnswer, REGULAR);
          if (i = 10) or (i = 70) or (i = 130) or (i = 190) or (i = 220) then
            ProcessAnswer(C2, TempAnswer, REVERSE);

          // N3
          if (i = 41) or (i = 101) or (i = 131) or (i = 161) or (i = 191) or (i = 221) then
            ProcessAnswer(N3, TempAnswer, REGULAR);
          if (i = 11) or (i = 71) then
            ProcessAnswer(N3, TempAnswer, REVERSE);
          // E3
          if (i = 12) or (i = 72) or (i = 132) or (i = 192) then
            ProcessAnswer(E3, TempAnswer, REGULAR);
          if (i = 42) or (i = 102) or (i = 162) or (i = 222) then
            ProcessAnswer(E3, TempAnswer, REVERSE);
          // O3
          if (i = 13) or (i = 73) or (i = 133) or (i = 193) or (i = 223) then
            ProcessAnswer(O3, TempAnswer, REGULAR);
          if (i = 43) or (i = 103) or (i = 163) then
            ProcessAnswer(O3, TempAnswer, REVERSE);
          // A3
          if (i = 44) or (i = 104) or (i = 164) or (i = 194) or (i = 224) then
            ProcessAnswer(A3, TempAnswer, REGULAR);
          if (i = 14) or (i = 74) or (i = 134) then
            ProcessAnswer(A3, TempAnswer, REVERSE);
          // C3
          if (i = 15) or (i = 75) or (i = 135) or (i = 165) or (i = 195) or (i = 225) then
            ProcessAnswer(C3, TempAnswer, REGULAR);
          if (i = 45) or (i = 105) then
            ProcessAnswer(C3, TempAnswer, REVERSE);

          // N4
          if (i = 16) or (i = 76) or (i = 136) or (i = 196) or (i = 226) then
            ProcessAnswer(N4, TempAnswer, REGULAR);
          if (i = 46) or (i = 106) or (i = 166) then
            ProcessAnswer(N4, TempAnswer, REVERSE);
          // E4
          if (i = 47) or (i = 107) or (i = 167) or (i = 197) or (i = 227) then
            ProcessAnswer(E4, TempAnswer, REGULAR);
          if (i = 17) or (i = 77) or (i = 137) then
            ProcessAnswer(E4, TempAnswer, REVERSE);
          // O4
          if (i = 48) or (i = 108) or (i = 168) then
            ProcessAnswer(O4, TempAnswer, REGULAR);
          if (i = 18) or (i = 78) or (i = 138) or (i = 198) or (i = 228) then
            ProcessAnswer(O4, TempAnswer, REVERSE);
          // A4
          if (i = 19) or (i = 79) or (i = 139) then
            ProcessAnswer(A4, TempAnswer, REGULAR);
          if (i = 49) or (i = 109) or (i = 169) or (i = 199) or (i = 229) then
            ProcessAnswer(A4, TempAnswer, REVERSE);
          // C4
          if (i = 50) or (i = 110) or (i = 170) or (i = 200) or (i = 230) then
            ProcessAnswer(C4, TempAnswer, REGULAR);
          if (i = 20) or (i = 80) or (i = 140) then
            ProcessAnswer(C4, TempAnswer, REVERSE);

          // N5
          if (i = 51) or (i = 111) or (i = 171) or (i = 201) then
            ProcessAnswer(N5, TempAnswer, REGULAR);
          if (i = 21) or (i = 81) or (i = 141) or (i = 231) then
            ProcessAnswer(N5, TempAnswer, REVERSE);
          // E5
          if (i = 22) or (i = 82) or (i = 142) or (i = 172) or (i = 202) or (i = 232) then
            ProcessAnswer(E5, TempAnswer, REGULAR);
          if (i = 52) or (i = 112) then
            ProcessAnswer(E5, TempAnswer, REVERSE);
          // O5
          if (i = 23) or (i = 83) or (i = 143) or (i = 203) or (i = 233) then
            ProcessAnswer(O5, TempAnswer, REGULAR);
          if (i = 53) or (i = 113) or (i = 173) then
            ProcessAnswer(O5, TempAnswer, REVERSE);
          // A5
          if (i = 54) or (i = 114) or (i = 174) or (i = 204) then
            ProcessAnswer(A5, TempAnswer, REGULAR);
          if (i = 24) or (i = 84) or (i = 144) or (i = 234) then
            ProcessAnswer(A5, TempAnswer, REVERSE);
          // C5
          if (i = 25) or (i = 85) or (i = 145) or (i = 235) then
            ProcessAnswer(C5, TempAnswer, REGULAR);
          if (i = 55) or (i = 115) or (i = 175) or (i = 205) then
            ProcessAnswer(C5, TempAnswer, REVERSE);

          // N6
          if (i = 26) or (i = 86) or (i = 146) then
            ProcessAnswer(N6, TempAnswer, REGULAR);
          if (i = 56) or (i = 116) or (i = 176) or (i = 206) or (i = 236) then
            ProcessAnswer(N6, TempAnswer, REVERSE);
          // E6
          if (i = 57) or (i = 117) or (i = 177) or (i = 237)  then
            ProcessAnswer(E6, TempAnswer, REGULAR);
          if (i = 27) or (i = 87) or (i = 147) or (i = 207) then
            ProcessAnswer(E6, TempAnswer, REVERSE);
          // O6
          if (i = 58) or (i = 118) or (i = 178) then
            ProcessAnswer(O6, TempAnswer, REGULAR);
          if (i = 28) or (i = 88) or (i = 148) or (i = 208) or (i = 238) then
            ProcessAnswer(O6, TempAnswer, REVERSE);
          // A6
          if (i = 29) or (i = 89) or (i = 149) or (i = 179) or (i = 209) or (i = 239) then
            ProcessAnswer(A6, TempAnswer, REGULAR);
          if (i = 59) or (i = 119) then
            ProcessAnswer(A6, TempAnswer, REVERSE);
          // C6
          if (i = 60) or (i = 120) or (i = 180) or (i = 210) or (i = 240) then
            ProcessAnswer(C6, TempAnswer, REGULAR);
          if (i = 30) or (i = 90) or (i = 150) then
            ProcessAnswer(C6, TempAnswer, REVERSE);

          // Three validity questions
          if (i = 241) then
            VA := TempAnswer;
          if (i = 242) then
            VB := TempAnswer;
          if (i = 243) then
            VC := TempAnswer;
        end; // for i
    end;

  N := N1 + N2 + N3 + N4 + N5 + N6;
  E := E1 + E2 + E3 + E4 + E5 + E6;
  O := O1 + O2 + O3 + O4 + O5 + O6;
  A := A1 + A2 + A3 + A4 + A5 + A6;
  C := C1 + C2 + C3 + C4 + C5 + C6;

 // Remember that the names of the scales has to be the same as the names used
 // in the editor (IAT).
  Result :=
    'N1: Anxiety^' + IntToStr(N1) + '^' + IntToStr(GetTScore('N1', N1)) + RECORD_END +
    'N2: Angry Hostility^' + IntToStr(N2) + '^' + IntToStr(GetTScore('N2', N2)) + RECORD_END +
    'N3: Depression^' + IntToStr(N3) + '^' + IntToStr(GetTScore('N3', N3)) + RECORD_END +
    'N4: Self-Consciousness^' + IntToStr(N4) + '^' + IntToStr(GetTScore('N4', N4)) + RECORD_END +
    'N5: Impulsiveness^' + IntToStr(N5) + '^' + IntToStr(GetTScore('N5', N5)) + RECORD_END +
    'N6: Vulnerability^' + IntToStr(N6) + '^' + IntToStr(GetTScore('N6', N6)) + RECORD_END +
    'Neuroticism^' + IntToStr(N) + '^' + IntToStr(GetTScore('N', N)) + RECORD_END +
    'E1: Warmth^' + IntToStr(E1) + '^' + IntToStr(GetTScore('E1', E1)) + RECORD_END +
    'E2: Gregariousness^' + IntToStr(E2) + '^' + IntToStr(GetTScore('E2', E2)) + RECORD_END +
    'E3: Assertiveness^' + IntToStr(E3) + '^' + IntToStr(GetTScore('E3', E3)) + RECORD_END +
    'E4: Activity^' + IntToStr(E4) + '^' + IntToStr(GetTScore('E4', E4)) + RECORD_END +
    'E5: Excitement-Seeking^' + IntToStr(E5) + '^' + IntToStr(GetTScore('E5', E5)) + RECORD_END +
    'E6: Positive Emotions^' + IntToStr(E6) + '^' + IntToStr(GetTScore('E6', E6)) + RECORD_END +
    'Extraversion^' + IntToStr(E) + '^' + IntToStr(GetTScore('E', E)) + RECORD_END +
    'O1: Fantasy^' + IntToStr(O1) + '^' + IntToStr(GetTScore('O1', O1)) + RECORD_END +
    'O2: Aesthetics^' + IntToStr(O2) + '^' + IntToStr(GetTScore('O2', O2)) + RECORD_END +
    'O3: Feelings^' + IntToStr(O3) + '^' + IntToStr(GetTScore('O3', O3)) + RECORD_END +
    'O4: Actions^' + IntToStr(O4) + '^' + IntToStr(GetTScore('O4', O4)) + RECORD_END +
    'O5: Ideas^' + IntToStr(O5) + '^' + IntToStr(GetTScore('O5', O5)) + RECORD_END +
    'O6: Values^' + IntToStr(O6) + '^' + IntToStr(GetTScore('O6', O6)) + RECORD_END +
    'Openness^' + IntToStr(O) + '^' + IntToStr(GetTScore('O', O)) + RECORD_END +
    'A1: Trust^' + IntToStr(A1) + '^' + IntToStr(GetTScore('A1', A1)) + RECORD_END +
    'A2: Straightforwardness^' + IntToStr(A2) + '^' + IntToStr(GetTScore('A2', A2)) + RECORD_END +
    'A3: Altruism^' + IntToStr(A3) + '^' + IntToStr(GetTScore('A3', A3)) + RECORD_END +
    'A4: Compliance^' + IntToStr(A4) + '^' + IntToStr(GetTScore('A4', A4)) + RECORD_END +
    'A5: Modesty^' + IntToStr(A5) + '^' + IntToStr(GetTScore('A5', A5)) + RECORD_END +
    'A6: Tender-Mindedness^' + IntToStr(A6) + '^' + IntToStr(GetTScore('A6', A6)) + RECORD_END +
    'Agreeableness^' + IntToStr(A) + '^' + IntToStr(GetTScore('A', A)) + RECORD_END +
    'C1: Competence^' + IntToStr(C1) + '^' + IntToStr(GetTScore('C1', C1)) + RECORD_END +
    'C2: Order^' + IntToStr(C2) + '^' + IntToStr(GetTScore('C2', C2)) + RECORD_END +
    'C3: Dutifulness^' + IntToStr(C3) + '^' + IntToStr(GetTScore('C3', C3)) + RECORD_END +
    'C4: Achievement Striving^' + IntToStr(C4) + '^' + IntToStr(GetTScore('C4', C4)) + RECORD_END +
    'C5: Self-Discipline^' + IntToStr(C5) + '^' + IntToStr(GetTScore('C5', C5)) + RECORD_END +
    'C6: Deliberation^' + IntToStr(C6) + '^' + IntToStr(GetTScore('C6', C6)) + RECORD_END +
    'Conscientiousness^' + IntToStr(C) + '^' + IntToStr(GetTScore('C', C)) + RECORD_END +
    'Skip^' + IntToStr(Skip) + RECORD_END +
    'Zero^' + IntToStr(Zero) + RECORD_END +
    'One^' + IntToStr(One) + RECORD_END +
    'Two^' + IntToStr(Two) + RECORD_END +
    'Three^' + IntToStr(Three) + RECORD_END +
    'Four^' + IntToStr(Four) + RECORD_END +
    'VA^' + VA + RECORD_END +
    'VB^' + VB + RECORD_END +
    'VC^' + VC + RECORD_END +
    'VR^' + VR;

end;


function GetRangeText(TScore: integer): string;
begin
  Result := '';

  if TScore = -1 then
    exit;

  if TScore > 65 then
    result := 'Very High';
  if (TScore <= 65) and (TScore > 55) then
    result := 'High';
  if (TScore <= 55) and (TScore >= 45) then
    result := 'Average';
  if (TScore < 45) and (TScore > 35) then
    result := 'Low';
  if (TScore <= 35) then
    result := 'Very Low';
end;


function ProgressNote_NEOPI3(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer, questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  i, j, x                     : Integer;
  VR,
    TempStr1,
    TempStr2                  : string;
  Skip, VA, VB, VC,
    Zero, One, Two, Three, Four,
    N, N1, N2, N3, N4, N5, N6,
    E, E1, E2, E3, E4, E5, E6,
    O, O1, O2, O3, O4, O5, O6,
    A, A1, A2, A3, A4, A5, A6,
    C, C1, C2, C3, C4, C5, C6 : Integer;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  N1 := -1;
  N2 := -1;
  N3 := -1;
  N4 := -1;
  N5 := -1;
  N6 := -1;
  E1 := -1;
  E2 := -1;
  E3 := -1;
  E4 := -1;
  E5 := -1;
  E6 := -1;
  O1 := -1;
  O2 := -1;
  O3 := -1;
  O4 := -1;
  O5 := -1;
  O6 := -1;
  A1 := -1;
  A2 := -1;
  A3 := -1;
  A4 := -1;
  A5 := -1;
  A6 := -1;
  C1 := -1;
  C2 := -1;
  C3 := -1;
  C4 := -1;
  C5 := -1;
  C6 := -1;
  // 07/31/2012 changed -1 to 0 llh
  Skip := 0;
  Zero := -1;
  One := -1;
  Two := -1;
  Three := -1;
  Four := -1;
  VA := -1;
  VB := -1;
  VC := -1;
  VR := '';

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      // Get name of scale "N1"
      TempStr1 := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      TempStr1 := Tempstr1[1] + TempStr1[2];

      // Get raw score
      if TempStr1 = 'VR' then
      VR := Piece(Piece(ScoreString, RECORD_END, i), U, 2)
      else
      x := StrToIntDef(Piece(Piece(ScoreString, RECORD_END, i), U, 2), -1);

      if TempStr1 = 'N1' then
        N1 := x;
      if TempStr1 = 'N2' then
        N2 := x;
      if TempStr1 = 'N3' then
        N3 := x;
      if TempStr1 = 'N4' then
        N4 := x;
      if TempStr1 = 'N5' then
        N5 := x;
      if TempStr1 = 'N6' then
        N6 := x;
      if TempStr1 = 'Ne' then
        N := x;
      if TempStr1 = 'E1' then
        E1 := x;
      if TempStr1 = 'E2' then
        E2 := x;
      if TempStr1 = 'E3' then
        E3 := x;
      if TempStr1 = 'E4' then
        E4 := x;
      if TempStr1 = 'E5' then
        E5 := x;
      if TempStr1 = 'E6' then
        E6 := x;
      if TempStr1 = 'Ex' then
        E := x;
      if TempStr1 = 'O1' then
        O1 := x;
      if TempStr1 = 'O2' then
        O2 := x;
      if TempStr1 = 'O3' then
        O3 := x;
      if TempStr1 = 'O4' then
        O4 := x;
      if TempStr1 = 'O5' then
        O5 := x;
      if TempStr1 = 'O6' then
        O6 := x;
      if TempStr1 = 'Op' then
        O := x;
      if TempStr1 = 'A1' then
        A1 := x;
      if TempStr1 = 'A2' then
        A2 := x;
      if TempStr1 = 'A3' then
        A3 := x;
      if TempStr1 = 'A4' then
        A4 := x;
      if TempStr1 = 'A5' then
        A5 := x;
      if TempStr1 = 'A6' then
        A6 := x;
      if TempStr1 = 'Ag' then
        A := x;
      if TempStr1 = 'C1' then
        C1 := x;
      if TempStr1 = 'C2' then
        C2 := x;
      if TempStr1 = 'C3' then
        C3 := x;
      if TempStr1 = 'C4' then
        C4 := x;
      if TempStr1 = 'C5' then
        C5 := x;
      if TempStr1 = 'C6' then
        C6 := x;
      if TempStr1 = 'Co' then
        C := x;
      if TempStr1 = 'Sk' then
        Skip := x;
      if TempStr1 = 'Ze' then
        Zero := x;
      if TempStr1 = 'On' then
        One := x;
      if TempStr1 = 'Tw' then
        Two := x;
      if TempStr1 = 'Th' then
        Three := x;
      if TempStr1 = 'Fo' then
        Four := x;

      if TempStr1 = 'VA' then
        VA := x;
      if TempStr1 = 'VB' then
        VB := x;
      if TempStr1 = 'VC' then
        VC := x;

    end;

  Result := Result + #13 + #13 + ' NEO Personality Inventory-3 ' + #13 + #13;

  // Too many blank items; don't score the results
  if Skip > 40 then
    begin
      Result := Result + ' Too many questions were not answered and no report is possible.' + #13;
      Result := Result + ' The maximum allowed is 41; in this administration the number of skipped items was ' + IntToStr(Skip) + '.' + #13;
      exit;
    end;

  if VR <> '' then // Random responding
    begin
      Result := Result + ' There is a very strong indication that the respondent answered the ' + #13;
      Result := Result + ' items in a random fashion. This is not a valid administrtion.' + #13;
      Result := Result + ' ' + VR;
      exit;
    end;

  if (VA = 0) or (VA = 1) then // Question A (Validity 1)
    begin
      Result := Result + ' WARNING: The validity of the results is in question. ' + #13;
      Result := Result + ' The respondent said they did not answer the questions honestly or accurately.' + #13;
      Result := Result + ' The report provided below may be valid depending on why the respondent' + #13;
      Result := Result + ' answered "Question A" with "Strongly disagree" or "Disagree."' + #13 + #13;
    end;

  if (VB = 1) then // Question B (Validity 2)
    begin
      Result := Result + ' Validity concern:' + #13;
      Result := Result + ' The respondent said they did not answer all the questions.' + #13;
      Result := Result + ' There are ' + inttostr(Skip) + ' skipped questions. If there are' + #13;
      Result := Result + ' more than 41 unanswered questions, the report is not valid.' + #13 + #13;
    end;

  if (VC = 1) then // Question C (Validity 3)
    begin
      Result := Result + ' Validity concern:' + #13;
      Result := Result + ' The respondent said they did not enter all their answers in the correct location.' + #13;
      Result := Result + ' The report provided below may be valid depending on why the respondent' + #13;
      Result := Result + ' answered "Question C" with a "No."' + #13 + #13;
    end;

  if (Three + Four) > 150 then
    begin
      Result := Result + ' Validity concern, Acquiescence:' + #13;
      Result := Result + ' The respondent has agreed, or strong agreed, with more than 150 items.' + #13;
      Result := Result + ' This is very unusual; 99% of volunteer respondents did not agree with more' + #13;
      Result := Result + ' than 150 items. Interprete these data with caution.' + #13 + #13;
    end;

  if (Three + Four) <= 50 then
    begin
      Result := Result + ' Validity concern, Nay-Saying:' + #13;
      Result := Result + ' The respondent has agreed, or strong agreed, with less than 50 items.' + #13;
      Result := Result + ' This is very unusual; 99% of volunteer respondents agreed with more' + #13;
      Result := Result + ' than 50 items. Interprete these data with caution.' + #13 + #13;
    end;

  Result := Result + '                             RAW  T-Score  Range' + #13;
  Result := Result + ' Domains' + #13 + #13;

  Result := Result + ' N:  Neuroticism               ' +
    IntToStr(N) + '  ' + IntToStr(GetTScore('N', N)) + '  ' + GetRangeText(GetTScore('N', N)) + #13;
  Result := Result + ' E:  Extraversion              ' +
    IntToStr(E) + '  ' + IntToStr(GetTScore('E', E)) + '  ' + GetRangeText(GetTScore('E', E)) + #13;
  Result := Result + ' O:  Openness                  ' +
    IntToStr(O) + '  ' + IntToStr(GetTScore('O', O)) + '  ' + GetRangeText(GetTScore('O', O)) + #13;
  Result := Result + ' A:  Agreeableness             ' +
    IntToStr(A) + '  ' + IntToStr(GetTScore('A', A)) + '  ' + GetRangeText(GetTScore('A', A)) + #13;
  Result := Result + ' C:  Conscientiousness         ' +
    IntToStr(C) + '  ' + IntToStr(GetTScore('C', C)) + '  ' + GetRangeText(GetTScore('C', C)) + #13 + #13;

  Result := Result + ' Neuroticism Facets' + #13 + #13;

  Result := Result + ' N1: Anxiety                   ' +
    IntToStr(N1) + '  ' + IntToStr(GetTScore('N1', N1)) + '  ' + GetRangeText(GetTScore('N1', N1)) + #13;
  Result := Result + ' N2: Angry Hostility           ' +
    IntToStr(N2) + '  ' + IntToStr(GetTScore('N2', N2)) + '  ' + GetRangeText(GetTScore('N2', N2)) + #13;
  Result := Result + ' N3: Depression                ' +
    IntToStr(N3) + '  ' + IntToStr(GetTScore('N3', N3)) + '  ' + GetRangeText(GetTScore('N3', N3)) + #13;
  Result := Result + ' N4: Self-Consciousness        ' +
    IntToStr(N4) + '  ' + IntToStr(GetTScore('N4', N4)) + '  ' + GetRangeText(GetTScore('N4', N4)) + #13;
  Result := Result + ' N5: Impulsiveness             ' +
    IntToStr(N5) + '  ' + IntToStr(GetTScore('N5', N5)) + '  ' + GetRangeText(GetTScore('N5', N5)) + #13;
  Result := Result + ' N6: Vulnerability             ' +
    IntToStr(N6) + '  ' + IntToStr(GetTScore('N6', N6)) + '  ' + GetRangeText(GetTScore('N6', N6)) + #13 + #13;

  Result := Result + ' Extraversion Facets' + #13 + #13;

  Result := Result + ' E1: Warmth                    ' +
    IntToStr(E1) + '  ' + IntToStr(GetTScore('E1', E1)) + '  ' + GetRangeText(GetTScore('E1', E1)) + #13;
  Result := Result + ' E2: Gregariousness            ' +
    IntToStr(E2) + '  ' + IntToStr(GetTScore('E2', E2)) + '  ' + GetRangeText(GetTScore('E2', E2)) + #13;
  Result := Result + ' E3: Assertiveness             ' +
    IntToStr(E3) + '  ' + IntToStr(GetTScore('E3', E3)) + '  ' + GetRangeText(GetTScore('E3', E3)) + #13;
  Result := Result + ' E4: Activity                  ' +
    IntToStr(E4) + '  ' + IntToStr(GetTScore('E4', E4)) + '  ' + GetRangeText(GetTScore('E4', E4)) + #13;
  Result := Result + ' E5: Excitement-Seeking        ' +
    IntToStr(E5) + '  ' + IntToStr(GetTScore('E5', E5)) + '  ' + GetRangeText(GetTScore('E5', E5)) + #13;
  Result := Result + ' E6: Positive Emotions         ' +
    IntToStr(E6) + '  ' + IntToStr(GetTScore('E6', E6)) + '  ' + GetRangeText(GetTScore('E6', E6)) + #13 + #13;

  Result := Result + ' Openness Facets' + #13 + #13;

  Result := Result + ' O1: Fantasy                   ' +
    IntToStr(O1) + '  ' + IntToStr(GetTScore('O1', O1)) + '  ' + GetRangeText(GetTScore('O1', O1)) + #13;
  Result := Result + ' O2: Aesthetics                ' +
    IntToStr(O2) + '  ' + IntToStr(GetTScore('O2', O2)) + '  ' + GetRangeText(GetTScore('O2', O2)) + #13;
  Result := Result + ' O3: Feelings                  ' +
    IntToStr(O3) + '  ' + IntToStr(GetTScore('O3', O3)) + '  ' + GetRangeText(GetTScore('O3', O3)) + #13;
  Result := Result + ' O4: Actions                   ' +
    IntToStr(O4) + '  ' + IntToStr(GetTScore('O4', O4)) + '  ' + GetRangeText(GetTScore('O4', O4)) + #13;
  Result := Result + ' O5: Ideas                     ' +
    IntToStr(O5) + '  ' + IntToStr(GetTScore('O5', O5)) + '  ' + GetRangeText(GetTScore('O5', O5)) + #13;
  Result := Result + ' O6: Values                    ' +
    IntToStr(O6) + '  ' + IntToStr(GetTScore('O6', O6)) + '  ' + GetRangeText(GetTScore('O6', O6)) + #13 + #13;

  Result := Result + ' Agreeableness Facets' + #13 + #13;

  Result := Result + ' A1: Trust                     ' +
    IntToStr(A1) + '  ' + IntToStr(GetTScore('A1', A1)) + '  ' + GetRangeText(GetTScore('A1', A1)) + #13;
  Result := Result + ' A2: Straightforwardness       ' +
    IntToStr(A2) + '  ' + IntToStr(GetTScore('A2', A2)) + '  ' + GetRangeText(GetTScore('A2', A2)) + #13;
  Result := Result + ' A3: Altruism                  ' +
    IntToStr(A3) + '  ' + IntToStr(GetTScore('A3', A3)) + '  ' + GetRangeText(GetTScore('A3', A3)) + #13;
  Result := Result + ' A4: Compliance                ' +
    IntToStr(A4) + '  ' + IntToStr(GetTScore('A4', A4)) + '  ' + GetRangeText(GetTScore('A4', A4)) + #13;
  Result := Result + ' A5: Modesty                   ' +
    IntToStr(A5) + '  ' + IntToStr(GetTScore('A5', A5)) + '  ' + GetRangeText(GetTScore('A5', A5)) + #13;
  Result := Result + ' A6: Tender-Mindedness         ' +
    IntToStr(A6) + '  ' + IntToStr(GetTScore('A6', A6)) + '  ' + GetRangeText(GetTScore('A6', A6)) + #13 + #13;

  Result := Result + ' Conscientiousness Facets' + #13 + #13;

  Result := Result + ' C1: Competence                ' +
    IntToStr(C1) + '  ' + IntToStr(GetTScore('C1', C1)) + '  ' + GetRangeText(GetTScore('C1', C1)) + #13;
  Result := Result + ' C2: Order                     ' +
    IntToStr(C2) + '  ' + IntToStr(GetTScore('C2', C2)) + '  ' + GetRangeText(GetTScore('C2', C2)) + #13;
  Result := Result + ' C3: Dutifulness               ' +
    IntToStr(C3) + '  ' + IntToStr(GetTScore('C3', C3)) + '  ' + GetRangeText(GetTScore('C3', C3)) + #13;
  Result := Result + ' C4: Achievement Striving      ' +
    IntToStr(C4) + '  ' + IntToStr(GetTScore('C4', C4)) + '  ' + GetRangeText(GetTScore('C4', C4)) + #13;
  Result := Result + ' C5: Self-Discipline           ' +
    IntToStr(C5) + '  ' + IntToStr(GetTScore('C5', C5)) + '  ' + GetRangeText(GetTScore('C5', C5)) + #13;
  Result := Result + ' C6: Deliberation              ' +
    IntToStr(C6) + '  ' + IntToStr(GetTScore('C6', C6)) + '  ' + GetRangeText(GetTScore('C6', C6)) + #13 + #13;

  Result := Result + ' Summary of Responses' + #13;
  Result := Result + '   SD: ' + FloatToStrF(100 * (Zero / 240), ffFixed, 15, 2) +
    '%   D: ' + FloatToStrF(100 * (One / 240), ffFixed, 15, 2) +
    '%   N: ' + FloatToStrF(100 * (Two / 240), ffFixed, 15, 2) +
    '%   A: ' + FloatToStrF(100 * (Three / 240), ffFixed, 15, 2) +
    '%   SA: ' + FloatToStrF(100 * (Four / 240), ffFixed, 15, 2) +
    '%   Skip: ' + IntToStr(Skip) + #13 + #13;

  Result := Result + ' Your NEO Summary' + #13 + #13;
  Result := Result + 'The NEO Inventory measures five broad domain, or dimensions, of personality.' + #13;
  Result := Result + 'Responses describing your thoughts, feelings, and goals can be compared with' + #13;
  Result := Result + 'those describing others'' to portray your personality.' + #13 + #13;
  Result := Result + 'This summary is intended to give you a general idea of what your personality' + #13;
  Result := Result + 'is like. It is not a detailed report. If you completed the inventory again,' + #13;
  Result := Result + 'or if someone else described you, you might score somewhat differently.' + #13 + #13;
  Result := Result + 'The NEO Inventory measures differences among people in general. It is not' + #13;
  Result := Result + 'a test of intelligence or ability, and it is not intended to diagnose' + #13;
  Result := Result + 'problems of mental health or adjustment. It does, however, give you some' + #13;
  Result := Result + 'idea of what makes you unique in your ways of thinking, feeling and' + #13;
  Result := Result + 'interacting with others.' + #13 + #13;

  Result := Result + 'Compared with the responses of other people, your responses suggest that you' + #13;
  Result := Result + 'can be described as:' + #13 + #13;

  X := GetTScore('N', N);
  if X > 55 then
    Result := Result + 'Sensitive, emotional, and prone to experience feelings that are upsetting.' + #13 + #13;
  if (X > 45) and (X <= 55) then
    begin
      Result := Result + 'Generally calm and able to deal with stress, but you sometimes experience' + #13;
      Result := Result + 'feelings of guilt, anger, or sadness.' + #13 + #13;
    end;
  if X <= 45 then
    Result := Result + 'Secure, hardy, and generally relaxed, even under stressful conditions.' + #13 + #13;

  X := GetTScore('E', E);
  if X > 55 then
    begin
      Result := Result + 'Extraverted, outgoing, active, and high-spirited. You prefer to be around' + #13;
      Result := Result + 'people most of the time.' + #13 + #13;
    end;
  if (X > 45) and (X <= 55) then
    begin
      Result := Result + 'Moderate in activity and enthusiasm. You enjoy the company of others,' + #13;
      Result := Result + 'but you also value privacy.' + #13 + #13;
    end;
  if X <= 45 then
    begin
      Result := Result + 'Introverted, reserved, and serious. You prefer to be alone or with' + #13;
      Result := Result + 'a few close friends.' + #13 + #13;
    end;

  X := GetTScore('O', O);
  if X > 55 then
    Result := Result + 'Open to new experiences. You have broad interests and are very imaginative.' + #13 + #13;
  if (X > 45) and (X <= 55) then
    begin
      Result := Result + 'Practical, but willing to consider new ways of doing things. You' + #13;
      Result := Result + 'seek a balance between the old and the new.' + #13 + #13;
    end;
  if X <= 45 then
    Result := Result + 'Down-to-earth, practical, traditional, and pretty much set in your ways.' + #13 + #13;

  X := GetTScore('A', A);
  if X > 55 then
    Result := Result + 'Compassionate, good-natured, and eager to cooperate and avoid conflict.' + #13 + #13;
  if (X > 45) and (X <= 55) then
    begin
      Result := Result + 'Generally warm, trusting and agreeable, but you can sometimes' + #13;
      Result := Result + 'be stubborn and competitive.' + #13 + #13;
    end;
  if X <= 45 then
    begin
      Result := Result + 'Hardheaded, skepitcal, proud, and competitive. You tend to exppress your' + #13;
      Result := Result + 'anger directly.' + #13 + #13;
    end;

  X := GetTScore('C', C);
  if X > 55 then
    begin
      Result := Result + 'Conscientious and well-organized. You have high standards and always' + #13;
      Result := Result + 'strive to achieve your goals.' + #13 + #13;
    end;
  if (X > 45) and (X <= 55) then
    begin
      Result := Result + 'Dependable and moderately well-organized. You generally have clear goals,' + #13;
      Result := Result + 'but are able to set your work aside.' + #13 + #13;
    end;
  if X <= 45 then
    begin
      Result := Result + 'Easygoing, not very well-organized, and sometimes careless. You prefer not' + #13;
      Result := Result + 'to make plans.' + #13 + #13;
    end;

  Result := Result + ' Copyright (c) 1978, 1985, 1989, 1991, 1992, 2010 by PAR. All rights reserved.' + #13 + #13;
end;

end.

